=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2006-02-21 22:41:44 +0000
+++ bzrlib/branch.py	2006-03-05 11:48:54 +0000
@@ -183,6 +183,14 @@
         """
         raise NotImplementedError('abspath is abstract')
 
+    def bind(self, other):
+        """Bind the local branch the other branch.
+
+        :param other: The branch to bind to
+        :type other: Branch
+        """
+        raise errors.UpgradeRequired(self.base)
+
     @needs_write_lock
     def fetch(self, from_branch, last_revision=None, pb=None):
         """Copy revisions from from_branch into this branch.
@@ -217,6 +225,21 @@
         finally:
             from_branch.unlock()
 
+    def get_bound_location(self):
+        """Return the URL of the rbanch we are bound to.
+
+        Older format branches cannot bind, please be sure to use a metadir
+        branch.
+        """
+        return None
+
+    def get_master_branch(self):
+        """Return the branch we are bound to.
+        
+        :return: Either a Branch, or None
+        """
+        return None
+
     def get_root_id(self):
         """Return the id of this branches root"""
         raise NotImplementedError('get_root_id is abstract')
@@ -243,6 +266,10 @@
         """
         return len(self.revision_history())
 
+    def unbind(self):
+        """Older format branches cannot bind or unbind."""
+        raise errors.UpgradeRequired(self.base)
+
     def last_revision(self):
         """Return last patch hash, or None if no history."""
         ph = self.revision_history()
@@ -251,7 +278,7 @@
         else:
             return None
 
-    def missing_revisions(self, other, stop_revision=None, diverged_ok=False):
+    def missing_revisions(self, other, stop_revision=None):
         """Return a list of new revisions that would perfectly fit.
         
         If self and other have not diverged, return a list of the revisions
@@ -301,7 +328,12 @@
         return other_history[self_len:stop_revision]
 
     def update_revisions(self, other, stop_revision=None):
-        """Pull in new perfect-fit revisions."""
+        """Pull in new perfect-fit revisions.
+
+        :param other: Another Branch to pull from
+        :param stop_revision: Updated until the given revision
+        :return: None
+        """
         raise NotImplementedError('update_revisions is abstract')
 
     def pullable_revisions(self, other, stop_revision):
@@ -380,6 +412,14 @@
     def set_parent(self, url):
         raise NotImplementedError('set_parent is abstract')
 
+    @needs_write_lock
+    def update(self):
+        """Synchronise this branch with the master branch if any. 
+
+        :return: None or the last_revision pivoted out during the update.
+        """
+        return None
+
     def check_revno(self, revno):
         """\
         Check whether a revno corresponds to any revision.
@@ -656,10 +696,13 @@
             assert format.__class__ == self.__class__
         transport = a_bzrdir.get_branch_transport(None)
         control_files = LockableFiles(transport, 'lock')
-        return BzrBranch(_format=self,
-                         _control_files=control_files,
-                         a_bzrdir=a_bzrdir,
-                         _repository=a_bzrdir.find_repository())
+        return BzrBranch5(_format=self,
+                          _control_files=control_files,
+                          a_bzrdir=a_bzrdir,
+                          _repository=a_bzrdir.find_repository())
+
+    def __str__(self):
+        return "Bazaar-NG Metadir branch format 5"
 
 
 class BranchReferenceFormat(BranchFormat):
@@ -744,17 +787,7 @@
     really matter if it's on an nfs/smb/afs/coda/... share, as long as
     it's writable, and can be accessed via the normal filesystem API.
     """
-    # We actually expect this class to be somewhat short-lived; part of its
-    # purpose is to try to isolate what bits of the branch logic are tied to
-    # filesystem access, so that in a later step, we can extricate them to
-    # a separarte ("storage") class.
-    _inventory_weave = None
     
-    # Map some sort of prefix into a namespace
-    # stuff like "revno:10", "revid:", etc.
-    # This should match a prefix with a function which accepts
-    REVISION_NAMESPACES = {}
-
     def __init__(self, transport=DEPRECATED_PARAMETER, init=DEPRECATED_PARAMETER,
                  relax_version_check=DEPRECATED_PARAMETER, _format=None,
                  _control_files=None, a_bzrdir=None, _repository=None):
@@ -901,7 +934,7 @@
         # TODO: test for failed two phase locks. This is known broken.
         self.repository.unlock()
         self.control_files.unlock()
-
+        
     def peek_lock_mode(self):
         if self.control_files._lock_count == 0:
             return None
@@ -979,7 +1012,6 @@
             self.append_revision(*pullable_revs)
 
     def pullable_revisions(self, other, stop_revision):
-        """See Branch.pullable_revisions."""
         other_revno = other.revision_id_to_revno(stop_revision)
         try:
             return self.missing_revisions(other, other_revno)
@@ -1063,39 +1095,141 @@
     def tree_config(self):
         return TreeConfig(self)
 
-    def _get_truncated_history(self, revision_id):
-        history = self.revision_history()
-        if revision_id is None:
-            return history
+
+class BzrBranch5(BzrBranch):
+    """A format 5 branch. This supports new features over plan branches.
+
+    It has support for a master_branch which is the data for bound branches.
+    """
+
+    def __init__(self,
+                 _format,
+                 _control_files,
+                 a_bzrdir,
+                 _repository):
+        super(BzrBranch5, self).__init__(_format=_format,
+                                         _control_files=_control_files,
+                                         a_bzrdir=a_bzrdir,
+                                         _repository=_repository)
+        
+    @needs_write_lock
+    def pull(self, source, overwrite=False, stop_revision=None):
+        """Updates branch.pull to be bound branch aware."""
+        bound_location = self.get_bound_location()
+        if source.base != bound_location:
+            # not pulling from master, so we need to update master.
+            master_branch = self.get_master_branch()
+            if master_branch:
+                master_branch.pull(source)
+                source = master_branch
+        return super(BzrBranch5, self).pull(source, overwrite, stop_revision)
+
+    def get_bound_location(self):
         try:
-            idx = history.index(revision_id)
-        except ValueError:
-            raise InvalidRevisionId(revision_id=revision, branch=self)
-        return history[:idx+1]
+            return self.control_files.get_utf8('bound').read()[:-1]
+        except errors.NoSuchFile:
+            return None
 
     @needs_read_lock
-    def _clone_weave(self, to_location, revision=None, basis_branch=None):
-        # prevent leakage
-        from bzrlib.workingtree import WorkingTree
-        assert isinstance(to_location, basestring)
-        if basis_branch is not None:
-            note("basis_branch is not supported for fast weave copy yet.")
-
-        history = self._get_truncated_history(revision)
-        if not bzrlib.osutils.lexists(to_location):
-            os.mkdir(to_location)
-        bzrdir_to = self.bzrdir._format.initialize(to_location)
-        self.repository.clone(bzrdir_to)
-        branch_to = bzrdir_to.create_branch()
-        mutter("copy branch from %s to %s", self, branch_to)
-
-        # FIXME duplicate code with base .clone().
-        # .. would template method be useful here?  RBC 20051207
-        branch_to.set_parent(self.base)
-        branch_to.append_revision(*history)
-        WorkingTree.create(branch_to, branch_to.base)
-        mutter("copied")
-        return branch_to
+    def get_master_branch(self):
+        """Return the branch we are bound to.
+        
+        :return: Either a Branch, or None
+
+        This could memoise the branch, but if thats done
+        it must be revalidated on each new lock.
+        So for now we just dont memoise it.
+        # RBC 20060304 review this decision.
+        """
+        bound_loc = self.get_bound_location()
+        if not bound_loc:
+            return None
+        try:
+            return Branch.open(bound_loc)
+        except (errors.NotBranchError, errors.ConnectionError), e:
+            raise errors.BoundBranchConnectionFailure(
+                    self, bound_loc, e)
+
+    @needs_write_lock
+    def set_bound_location(self, location):
+        """Set the target where this branch is bound to.
+
+        :param location: URL to the target branch
+        """
+        if location:
+            self.control_files.put_utf8('bound', location+'\n')
+        else:
+            try:
+                self.control_files._transport.delete('bound')
+            except NoSuchFile:
+                return False
+            return True
+
+    @needs_write_lock
+    def bind(self, other):
+        """Bind the local branch the other branch.
+
+        :param other: The branch to bind to
+        :type other: Branch
+        """
+        # TODO: jam 20051230 Consider checking if the target is bound
+        #       It is debatable whether you should be able to bind to
+        #       a branch which is itself bound.
+        #       Committing is obviously forbidden,
+        #       but binding itself may not be.
+        #       Since we *have* to check at commit time, we don't
+        #       *need* to check here
+        self.pull(other)
+
+        # we are now equal to or a suffix of other.
+
+        # Since we have 'pulled' from the remote location,
+        # now we should try to pull in the opposite direction
+        # in case the local tree has more revisions than the
+        # remote one.
+        # There may be a different check you could do here
+        # rather than actually trying to install revisions remotely.
+        # TODO: capture an exception which indicates the remote branch
+        #       is not writeable. 
+        #       If it is up-to-date, this probably should not be a failure
+        
+        # lock other for write so the revision-history syncing cannot race
+        other.lock_write()
+        try:
+            other.pull(self)
+            # if this does not error, other now has the same last rev we do
+            # it can only error if the pull from other was concurrent with
+            # a commit to other from someone else.
+
+            # until we ditch revision-history, we need to sync them up:
+            self.set_revision_history(other.revision_history())
+            # now other and self are up to date with each other and have the
+            # same revision-history.
+        finally:
+            other.unlock()
+
+        self.set_bound_location(other.base)
+
+    @needs_write_lock
+    def unbind(self):
+        """If bound, unbind"""
+        return self.set_bound_location(None)
+
+    @needs_write_lock
+    def update(self):
+        """Synchronise this branch with the master branch if any. 
+
+        :return: None or the last_revision that was pivoted out during the
+                 update.
+        """
+        master = self.get_master_branch()
+        if master is not None:
+            old_tip = self.last_revision()
+            self.pull(master, overwrite=True)
+            if old_tip in self.repository.get_ancestry(self.last_revision()):
+                return None
+            return old_tip
+        return None
 
 
 class BranchTestProviderAdapter(object):

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-03-03 05:38:04 +0000
+++ bzrlib/builtins.py	2006-03-05 11:48:54 +0000
@@ -596,6 +596,7 @@
                 else:
                     raise
             try:
+                # preserve whatever source format we have.
                 dir = br_from.bzrdir.sprout(to_location, revision_id, basis_dir)
                 branch = dir.open_branch()
             except bzrlib.errors.NoSuchRevision:
@@ -604,7 +605,7 @@
                 raise BzrCommandError(msg)
             except bzrlib.errors.UnlistableBranch:
                 rmtree(to_location)
-                msg = "The branch %s cannot be used as a --basis"
+                msg = "The branch %s cannot be used as a --basis" % (basis,)
                 raise BzrCommandError(msg)
             if name:
                 branch.control_files.put_utf8('branch-name', name)
@@ -630,9 +631,18 @@
     branch being checked out. [Not implemented yet.]
     """
     takes_args = ['branch_location', 'to_location?']
-    takes_options = ['revision'] # , 'basis']
+    takes_options = ['revision', # , 'basis']
+                     Option('lightweight',
+                            help="perform a lightweight checkout. Lightweight "
+                                 "checkouts depend on access to the branch for "
+                                 "every operation. Normal checkouts can perform "
+                                 "common operations like diff and status without "
+                                 "such access, and also support local commits."
+                            ),
+                     ]
 
-    def run(self, branch_location, to_location=None, revision=None, basis=None):
+    def run(self, branch_location, to_location=None, revision=None, basis=None,
+            lightweight=False):
         if revision is None:
             revision = [None]
         elif len(revision) > 1:
@@ -656,9 +666,23 @@
                                       to_location)
             else:
                 raise
-        checkout = bzrdir.BzrDirMetaFormat1().initialize(to_location)
-        bzrlib.branch.BranchReferenceFormat().initialize(checkout, source)
-        checkout.create_workingtree(revision_id)
+        old_format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
+        bzrlib.bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirMetaFormat1())
+        try:
+            if lightweight:
+                checkout = bzrdir.BzrDirMetaFormat1().initialize(to_location)
+                bzrlib.branch.BranchReferenceFormat().initialize(checkout, source)
+            else:
+                checkout_branch =  bzrlib.bzrdir.BzrDir.create_branch_convenience(
+                    to_location, force_new_tree=False)
+                checkout = checkout_branch.bzrdir
+                checkout_branch.bind(source)
+                if revision_id is not None:
+                    rh = checkout_branch.revision_history()
+                    checkout_branch.set_revision_history(rh[:rh.index(revision_id) + 1])
+            checkout.create_workingtree(revision_id)
+        finally:
+            bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)
 
 
 class cmd_renames(Command):
@@ -685,8 +709,11 @@
     """Update a tree to have the latest code committed to its branch.
     
     This will perform a merge into the working tree, and may generate
-    conflicts. If you have any uncommitted changes, you will still 
-    need to commit them after the update.
+    conflicts. If you have any local changes, you will still 
+    need to commit them after the update for the update to be complete.
+    
+    If you want to discard your local changes, you can just do a 
+    'bzr revert' instead of 'bzr commit' after the update.
     """
     takes_args = ['dir?']
 
@@ -695,8 +722,11 @@
         tree.lock_write()
         try:
             if tree.last_revision() == tree.branch.last_revision():
-                note("Tree is up to date.")
-                return
+                # may be up to date, check master too.
+                master = tree.branch.get_master_branch()
+                if master is None or master.last_revision == tree.last_revision():
+                    note("Tree is up to date.")
+                    return
             conflicts = tree.update()
             note('Updated to revision %d.' %
                  (tree.branch.revision_id_to_revno(tree.last_revision()),))
@@ -1385,11 +1415,17 @@
                      Option('strict',
                             help="refuse to commit if there are unknown "
                             "files in the working tree."),
+                     Option('local',
+                            help="perform a local only commit in a bound "
+                                 "branch. Such commits are not pushed to "
+                                 "the master branch until a normal commit "
+                                 "is performed."
+                            ),
                      ]
     aliases = ['ci', 'checkin']
 
     def run(self, message=None, file=None, verbose=True, selected_list=None,
-            unchanged=False, strict=False):
+            unchanged=False, strict=False, local=False):
         from bzrlib.errors import (PointlessCommit, ConflictsInTree,
                 StrictCommitFailed)
         from bzrlib.msgeditor import edit_commit_message, \
@@ -1406,6 +1442,8 @@
         # TODO: if the commit *does* happen to fail, then save the commit 
         # message to a temporary file where it can be recovered
         tree, selected_list = tree_files(selected_list)
+        if local and not tree.branch.get_bound_location():
+            raise errors.LocalRequiresBoundBranch()
         if message is None and not file:
             template = make_commit_message_template(tree, selected_list)
             message = edit_commit_message(template)
@@ -1424,7 +1462,7 @@
             
         try:
             tree.commit(message, specific_files=selected_list,
-                        allow_pointless=unchanged, strict=strict)
+                        allow_pointless=unchanged, strict=strict, local=local)
         except PointlessCommit:
             # FIXME: This should really happen before the file is read in;
             # perhaps prepare the commit; get the message; then actually commit
@@ -1436,6 +1474,11 @@
         except StrictCommitFailed:
             raise BzrCommandError("Commit refused because there are unknown "
                                   "files in the working tree.")
+        except errors.BoundBranchOutOfDate, e:
+            raise BzrCommandError(str(e)
+                                  + ' Either unbind, update, or'
+                                    ' pass --local to commit.')
+
         note('Committed revision %d.' % (tree.branch.revno(),))
 
 
@@ -2122,6 +2165,41 @@
                 raise BzrCommandError('Please supply either one revision, or a range.')
 
 
+class cmd_bind(Command):
+    """Bind the current branch to a master branch.
+
+    After binding, commits must succeed on the master branch
+    before they are executed on the local one.
+    """
+
+    takes_args = ['location']
+    takes_options = []
+
+    def run(self, location=None):
+        b, relpath = Branch.open_containing(u'.')
+        b_other = Branch.open(location)
+        try:
+            b.bind(b_other)
+        except DivergedBranches:
+            raise BzrCommandError('These branches have diverged.'
+                                  ' Try merging, and then bind again.')
+
+
+class cmd_unbind(Command):
+    """Bind the current branch to its parent.
+
+    After unbinding, the local branch is considered independent.
+    """
+
+    takes_args = []
+    takes_options = []
+
+    def run(self):
+        b, relpath = Branch.open_containing(u'.')
+        if not b.unbind():
+            raise BzrCommandError('Local branch is not bound')
+
+
 class cmd_uncommit(bzrlib.commands.Command):
     """Remove the last committed revision.
 

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2006-02-22 23:02:29 +0000
+++ bzrlib/bzrdir.py	2006-03-03 13:22:46 +0000
@@ -525,12 +525,7 @@
             source_branch.sprout(result, revision_id=revision_id)
         else:
             result.create_branch()
-        try:
-            self.open_workingtree().clone(result,
-                                          revision_id=revision_id, 
-                                          basis=basis_tree)
-        except (errors.NoWorkingTree, errors.NotLocalUrl):
-            result.create_workingtree()
+        result.create_workingtree()
         return result
 
 
@@ -641,11 +636,8 @@
             self.open_branch().sprout(result, revision_id=revision_id)
         except errors.NotBranchError:
             pass
-        try:
-            self.open_workingtree().clone(result, basis=basis_tree)
-        except (errors.NotBranchError, errors.NotLocalUrl):
-            # we always want a working tree
-            WorkingTreeFormat2().initialize(result)
+        # we always want a working tree
+        WorkingTreeFormat2().initialize(result)
         return result
 
 

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2006-02-23 04:08:56 +0000
+++ bzrlib/commit.py	2006-03-04 23:04:37 +0000
@@ -180,7 +180,8 @@
                strict=False,
                verbose=False,
                revprops=None,
-               working_tree=None):
+               working_tree=None,
+               local=False):
         """Commit working copy as a new revision.
 
         branch -- the deprecated branch to commit to. New callers should pass in 
@@ -206,6 +207,7 @@
             contains unknown files.
 
         revprops -- Properties for new revision
+        :param local: Perform a local only commit.
         """
         mutter('preparing to commit')
 
@@ -224,6 +226,9 @@
             raise BzrError("The message keyword parameter is required for commit().")
 
         self.weave_store = self.branch.repository.weave_store
+        self.bound_branch = None
+        self.local = local
+        self.master_branch = None
         self.rev_id = rev_id
         self.specific_files = specific_files
         self.allow_pointless = allow_pointless
@@ -231,47 +236,52 @@
         if revprops:
             self.revprops.update(revprops)
 
-        # check for out of date working trees
-        if self.work_tree.last_revision() != self.branch.last_revision():
-            raise errors.OutOfDateTree(self.work_tree)
-
-        if strict:
-            # raise an exception as soon as we find a single unknown.
-            for unknown in self.work_tree.unknowns():
-                raise StrictCommitFailed()
-
-        if timestamp is None:
-            self.timestamp = time.time()
-        else:
-            self.timestamp = long(timestamp)
-            
-        if self.config is None:
-            self.config = bzrlib.config.BranchConfig(self.branch)
-
-        if rev_id is None:
-            self.rev_id = _gen_revision_id(self.config, self.timestamp)
-        else:
-            self.rev_id = rev_id
-
-        if committer is None:
-            self.committer = self.config.username()
-        else:
-            assert isinstance(committer, basestring), type(committer)
-            self.committer = committer
-
-        if timezone is None:
-            self.timezone = local_time_offset()
-        else:
-            self.timezone = int(timezone)
-
-        if isinstance(message, str):
-            message = message.decode(bzrlib.user_encoding)
-        assert isinstance(message, unicode), type(message)
-        self.message = message
-        self._escape_commit_message()
-
-        self.branch.lock_write()
+        self.work_tree.lock_write()
         try:
+            # setup the bound branch variables as needed.
+            self._check_bound_branch()
+
+            # check for out of date working trees
+            # if we are bound, then self.branch is the master branch and this
+            # test is thus all we need.
+            if self.work_tree.last_revision() != self.master_branch.last_revision():
+                raise errors.OutOfDateTree(self.work_tree)
+    
+            if strict:
+                # raise an exception as soon as we find a single unknown.
+                for unknown in self.work_tree.unknowns():
+                    raise StrictCommitFailed()
+    
+            if timestamp is None:
+                self.timestamp = time.time()
+            else:
+                self.timestamp = long(timestamp)
+                
+            if self.config is None:
+                self.config = bzrlib.config.BranchConfig(self.branch)
+    
+            if rev_id is None:
+                self.rev_id = _gen_revision_id(self.config, self.timestamp)
+            else:
+                self.rev_id = rev_id
+    
+            if committer is None:
+                self.committer = self.config.username()
+            else:
+                assert isinstance(committer, basestring), type(committer)
+                self.committer = committer
+    
+            if timezone is None:
+                self.timezone = local_time_offset()
+            else:
+                self.timezone = int(timezone)
+    
+            if isinstance(message, str):
+                message = message.decode(bzrlib.user_encoding)
+            assert isinstance(message, unicode), type(message)
+            self.message = message
+            self._escape_commit_message()
+
             self.work_inv = self.work_tree.inventory
             self.basis_tree = self.work_tree.basis_tree()
             self.basis_inv = self.basis_tree.inventory
@@ -300,13 +310,29 @@
                 self.present_parents
                 )
             self._make_revision()
+            # revision data is in the local branch now.
+            
+            # upload revision data to the master.
+            # this will propogate merged revisions too if needed.
+            if self.bound_branch:
+                self.master_branch.repository.fetch(self.branch.repository,
+                                                    revision_id=self.rev_id)
+                # now the master has the revision data
+                # 'commit' to the master first so a timeout here causes the local
+                # branch to be out of date
+                self.master_branch.append_revision(self.rev_id)
+
+            # and now do the commit locally.
+            self.branch.append_revision(self.rev_id)
+
             self.work_tree.set_pending_merges([])
-            self.branch.append_revision(self.rev_id)
             if len(self.parents):
                 precursor = self.parents[0]
             else:
                 precursor = None
             self.work_tree.set_last_revision(self.rev_id, precursor)
+            # now the work tree is up to date with the branch
+            
             self.reporter.completed(self.branch.revno()+1, self.rev_id)
             if self.config.post_commit() is not None:
                 hooks = self.config.post_commit().split(' ')
@@ -317,7 +343,68 @@
                                    'bzrlib':bzrlib,
                                    'rev_id':self.rev_id})
         finally:
-            self.branch.unlock()
+            self._cleanup_bound_branch()
+            self.work_tree.unlock()
+
+    def _check_bound_branch(self):
+        """Check to see if the local branch is bound.
+
+        If it is bound, then most of the commit will actually be
+        done using the remote branch as the target branch.
+        Only at the end will the local branch be updated.
+        """
+        if self.local and not self.branch.get_bound_location():
+            raise errors.LocalRequiresBoundBranch()
+
+        if not self.local:
+            self.master_branch = self.branch.get_master_branch()
+
+        if not self.master_branch:
+            # make this branch the reference branch for out of date checks.
+            self.master_branch = self.branch
+            return
+
+        # If the master branch is bound, we must fail
+        master_bound_location = self.master_branch.get_bound_location()
+        if master_bound_location:
+            raise errors.CommitToDoubleBoundBranch(self.branch,
+                    self.master_branch, master_bound_location)
+
+        # TODO: jam 20051230 We could automatically push local
+        #       commits to the remote branch if they would fit.
+        #       But for now, just require remote to be identical
+        #       to local.
+        
+        # Make sure the local branch is identical to the master
+        master_rh = self.master_branch.revision_history()
+        local_rh = self.branch.revision_history()
+        if local_rh != master_rh:
+            raise errors.BoundBranchOutOfDate(self.branch,
+                    self.master_branch)
+
+        # Now things are ready to change the master branch
+        # so grab the lock
+        self.bound_branch = self.branch
+        self.master_branch.lock_write()
+####        
+####        # Check to see if we have any pending merges. If we do
+####        # those need to be pushed into the master branch
+####        pending_merges = self.work_tree.pending_merges()
+####        if pending_merges:
+####            for revision_id in pending_merges:
+####                self.master_branch.repository.fetch(self.bound_branch.repository,
+####                                                    revision_id=revision_id)
+
+    def _cleanup_bound_branch(self):
+        """Executed at the end of a try/finally to cleanup a bound branch.
+
+        If the branch wasn't bound, this is a no-op.
+        If it was, it resents self.branch to the local branch, instead
+        of being the master.
+        """
+        if not self.bound_branch:
+            return
+        self.master_branch.unlock()
 
     def _escape_commit_message(self):
         """Replace xml-incompatible control characters."""

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-02-26 07:54:02 +0000
+++ bzrlib/errors.py	2006-03-04 22:35:19 +0000
@@ -368,9 +368,11 @@
         self.format = format
 
 
+
 class StrictCommitFailed(Exception):
     """Commit refused because there are unknowns in the tree."""
 
+
 class NoSuchRevision(BzrError):
     def __init__(self, branch, revision):
         self.branch = branch
@@ -415,6 +417,7 @@
         BzrError.__init__(self, msg)
 
 
+
 class NotAncestor(BzrError):
     def __init__(self, rev_id, not_ancestor_id):
         msg = "Revision %s is not an ancestor of %s" % (not_ancestor_id, 
@@ -450,11 +453,45 @@
         BzrError.__init__(self, "Store %s is not listable" % store)
 
 
+
 class UnlistableBranch(BzrError):
     def __init__(self, br):
         BzrError.__init__(self, "Stores for branch %s are not listable" % br)
 
 
+class BoundBranchOutOfDate(BzrNewError):
+    """Bound branch %(branch)s is out of date with master branch %(master)s."""
+    def __init__(self, branch, master):
+        BzrNewError.__init__(self)
+        self.branch = branch
+        self.master = master
+
+        
+class CommitToDoubleBoundBranch(BzrNewError):
+    """Cannot commit to branch %(branch)s. It is bound to %(master)s, which is bound to %(remote)s."""
+    def __init__(self, branch, master, remote):
+        BzrNewError.__init__(self)
+        self.branch = branch
+        self.master = master
+        self.remote = remote
+
+
+class OverwriteBoundBranch(BzrNewError):
+    """Cannot pull --overwrite to a branch which is bound %(branch)s"""
+    def __init__(self, branch):
+        BzrNewError.__init__(self)
+        self.branch = branch
+
+
+class BoundBranchConnectionFailure(BzrNewError):
+    """Unable to connect to target of bound branch %(branch)s => %(target)s: %(error)s"""
+    def __init__(self, branch, target, error):
+        BzrNewError.__init__(self)
+        self.branch = branch
+        self.target = target
+        self.error = error
+
+
 class WeaveError(BzrNewError):
     """Error in processing weave: %(message)s"""
     def __init__(self, message=None):
@@ -618,9 +655,11 @@
         self.text_revision = text_revision
         self.file_id = file_id
 
+
 class DuplicateKey(BzrNewError):
     """Key %(key)s is already present in map"""
 
+
 class MalformedTransform(BzrNewError):
     """Tree transform is malformed %(conflicts)r"""
 
@@ -709,6 +748,7 @@
         BzrNewError.__init__(self)
         self.tree = tree
 
+
 class CorruptRepository(BzrNewError):
     """An error has been detected in the repository %(repo_path)s.
 Please run bzr reconcile on this repository."""
@@ -716,3 +756,15 @@
     def __init__(self, repo):
         BzrNewError.__init__(self)
         self.repo_path = repo.bzrdir.root_transport.base
+
+
+class UpgradeRequired(BzrNewError):
+    """To use this feature you must upgrade your branch at %(path)s."""
+
+    def __init__(self, path):
+        BzrNewError.__init__(self)
+        self.path = path
+
+
+class LocalRequiresBoundBranch(BzrNewError):
+    """Cannot perform local-only commits on unbound branches."""

=== modified file 'bzrlib/info.py'
--- bzrlib/info.py	2006-02-15 06:08:08 +0000
+++ bzrlib/info.py	2006-03-05 11:48:54 +0000
@@ -22,6 +22,7 @@
 
 
 import bzrlib.diff as diff
+from bzrlib.missing import find_unmerged
 from bzrlib.osutils import format_date
 from bzrlib.symbol_versioning import *
 
@@ -42,6 +43,14 @@
 def show_bzrdir_info(a_bzrdir):
     """Output to stdout the 'info' for a_bzrdir."""
 
+    def plural(n, base='', pl=None):
+        if n == 1:
+            return base
+        elif pl != None:
+            return pl
+        else:
+            return 's'
+
     working = a_bzrdir.open_workingtree()
     b = a_bzrdir.open_branch()
     
@@ -55,13 +64,8 @@
         format = b.bzrdir._format
     print 'branch format:', format
 
-    def plural(n, base='', pl=None):
-        if n == 1:
-            return base
-        elif pl != None:
-            return pl
-        else:
-            return 's'
+    if b.get_bound_location():
+        print 'bound to branch:',  b.get_bound_location()
 
     count_version_dirs = 0
 
@@ -71,6 +75,14 @@
     history = b.revision_history()
     
     print
+    # Try with inaccessible branch ?
+    master = b.get_master_branch()
+    if master:
+        local_extra, remote_extra = find_unmerged(b, b.get_master_branch())
+        if remote_extra:
+            print 'Branch is out of date: missing %d revision%s.' % (
+                len(remote_extra), plural(len(remote_extra)))
+
     if len(history) and working.last_revision() != history[-1]:
         try:
             missing_count = len(history) - history.index(working.last_revision())

=== modified file 'bzrlib/option.py'
--- bzrlib/option.py	2006-02-03 19:50:59 +0000
+++ bzrlib/option.py	2006-03-04 13:03:57 +0000
@@ -161,6 +161,7 @@
 _global_option('overwrite', help='Ignore differences between branches and '
                'overwrite unconditionally')
 _global_option('basis', type=str)
+_global_option('bound')
 _global_option('diff-options', type=str)
 _global_option('help',
                help='show help message')
@@ -179,6 +180,7 @@
 _global_option('timezone', 
                type=str,
                help='display timezone as local, original, or utc')
+_global_option('unbound')
 _global_option('verbose',
                help='display more information')
 _global_option('version')

=== modified file 'bzrlib/revision.py'
--- bzrlib/revision.py	2006-03-03 09:41:00 +0000
+++ bzrlib/revision.py	2006-03-05 02:20:07 +0000
@@ -259,6 +259,8 @@
 
 def common_ancestor(revision_a, revision_b, revision_source, 
                     pb=DummyProgress()):
+    if None in (revision_a, revision_b):
+        return None
     try:
         try:
             pb.update('Picking ancestor', 1, 3)

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2006-03-03 05:55:51 +0000
+++ bzrlib/tests/blackbox/__init__.py	2006-03-04 13:03:57 +0000
@@ -37,6 +37,7 @@
                      'bzrlib.tests.blackbox.test_added',
                      'bzrlib.tests.blackbox.test_aliases',
                      'bzrlib.tests.blackbox.test_ancestry',
+                     'bzrlib.tests.blackbox.test_bound_branches',
                      'bzrlib.tests.blackbox.test_cat',
                      'bzrlib.tests.blackbox.test_checkout',
                      'bzrlib.tests.blackbox.test_commit',

=== added file 'bzrlib/tests/blackbox/test_bound_branches.py'
--- bzrlib/tests/blackbox/test_bound_branches.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_bound_branches.py	2006-03-05 11:48:54 +0000
@@ -0,0 +1,391 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Tests of bound branches (binding, unbinding, commit, etc) command.
+"""
+
+import os
+from cStringIO import StringIO
+
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.branch import Branch
+from bzrlib.bzrdir import (BzrDir,
+                           BzrDirFormat,
+                           BzrDirFormat6,
+                           BzrDirMetaFormat1,
+                           )
+from bzrlib.workingtree import WorkingTree
+
+
+class TestLegacyFormats(TestCaseWithTransport):
+    
+    def setUp(self):
+        super(TestLegacyFormats, self).setUp()
+        self.build_tree(['master/', 'child/'])
+        self.run_bzr('init', 'master')
+        old_format = BzrDirFormat.get_default_format()
+        BzrDirFormat.set_default_format(BzrDirFormat6())
+        try:
+            self.run_bzr('init', 'child')
+        finally:
+            BzrDirFormat.set_default_format(old_format)
+        os.chdir('child')
+    
+    def test_bind_format_6_bzrdir(self):
+        # bind on a format 6 bzrdir should error
+        out,err = self.run_bzr('bind', '../master', retcode=3)
+        self.assertEqual('', out)
+        self.assertEqual('bzr: ERROR: To use this feature you must '
+                         'upgrade your branch at %s/.\n' % os.getcwdu(), err)
+    
+    def test_unbind_format_6_bzrdir(self):
+        # bind on a format 6 bzrdir should error
+        out,err = self.run_bzr('unbind', retcode=3)
+        self.assertEqual('', out)
+        self.assertEqual('bzr: ERROR: To use this feature you must '
+                         'upgrade your branch at %s/.\n' % os.getcwdu(), err)
+
+
+class TestBoundBranches(TestCaseWithTransport):
+
+    def create_branches(self):
+        bzr = self.run_bzr
+        self.build_tree(['base/', 'base/a', 'base/b'])
+
+        self.init_meta_branch('base')
+        os.chdir('base')
+        bzr('add')
+        bzr('commit', '-m', 'init')
+
+        os.chdir('..')
+
+        bzr('checkout', 'base', 'child')
+
+        self.failUnlessExists('child')
+
+        self.check_revno(1, 'child')
+        d = BzrDir.open('child')
+        self.assertNotEqual(None, d.open_branch().get_master_branch())
+
+    def check_revno(self, val, loc=None):
+        if loc is not None:
+            cwd = os.getcwd()
+            os.chdir(loc)
+        self.assertEquals(str(val), self.run_bzr('revno')[0].strip())
+        if loc is not None:
+            os.chdir(cwd)
+
+    def test_simple_binding(self):
+        self.build_tree(['base/', 'base/a', 'base/b'])
+
+        self.init_meta_branch('base')
+        self.run_bzr('add', 'base')
+        self.run_bzr('commit', '-m', 'init', 'base')
+
+        self.run_bzr('branch', 'base', 'child')
+
+        os.chdir('child')
+        self.run_bzr('bind', '../base')
+
+        d = BzrDir.open('')
+        self.assertNotEqual(None, d.open_branch().get_master_branch())
+
+        self.run_bzr('unbind')
+        self.assertEqual(None, d.open_branch().get_master_branch())
+
+        self.run_bzr('unbind', retcode=3)
+
+    def init_meta_branch(self, path):
+        old_format = BzrDirFormat.get_default_format()
+        BzrDirFormat.set_default_format(BzrDirMetaFormat1())
+        try:
+            self.run_bzr('init', path)
+        finally:
+            BzrDirFormat.set_default_format(old_format)
+
+    def test_bound_commit(self):
+        bzr = self.run_bzr
+        self.create_branches()
+
+        os.chdir('child')
+        open('a', 'wb').write('new contents\n')
+        bzr('commit', '-m', 'child')
+
+        self.check_revno(2)
+
+        # Make sure it committed on the parent
+        self.check_revno(2, '../base')
+
+    def test_bound_fail(self):
+        # Make sure commit fails if out of date.
+        bzr = self.run_bzr
+        self.create_branches()
+
+        os.chdir('base')
+        open('a', 'wb').write('new base contents\n')
+        bzr('commit', '-m', 'base')
+        self.check_revno(2)
+
+        os.chdir('../child')
+        self.check_revno(1)
+        open('b', 'wb').write('new b child contents\n')
+        bzr('commit', '-m', 'child', retcode=3)
+        self.check_revno(1)
+
+        bzr('update')
+        self.check_revno(2)
+
+        bzr('commit', '-m', 'child')
+        self.check_revno(3)
+        self.check_revno(3, '../base')
+
+    def test_double_binding(self):
+        bzr = self.run_bzr
+        self.create_branches()
+
+        bzr('branch', 'child', 'child2')
+        os.chdir('child2')
+
+        # Double binding succeeds, but committing to child2 should fail
+        bzr('bind', '../child')
+
+        bzr('commit', '-m', 'child2', '--unchanged', retcode=3)
+
+    def test_unbinding(self):
+        bzr = self.run_bzr
+        self.create_branches()
+
+        os.chdir('base')
+        open('a', 'wb').write('new base contents\n')
+        bzr('commit', '-m', 'base')
+        self.check_revno(2)
+
+        os.chdir('../child')
+        open('b', 'wb').write('new b child contents\n')
+        self.check_revno(1)
+        bzr('commit', '-m', 'child', retcode=3)
+        self.check_revno(1)
+        bzr('unbind')
+        bzr('commit', '-m', 'child')
+        self.check_revno(2)
+
+        bzr('bind', retcode=3)
+
+    def test_commit_remote_bound(self):
+        # It is not possible to commit to a branch
+        # which is bound to a branch which is bound
+        bzr = self.run_bzr
+        self.create_branches()
+        bzr('branch', 'base', 'newbase')
+        os.chdir('base')
+        
+        # There is no way to know that B has already
+        # been bound by someone else, otherwise it
+        # might be nice if this would fail
+        bzr('bind', '../newbase')
+
+        os.chdir('../child')
+        bzr('commit', '-m', 'failure', '--unchanged', retcode=3)
+
+    def test_pull_updates_both(self):
+        bzr = self.run_bzr
+        self.create_branches()
+        bzr('branch', 'base', 'newchild')
+        os.chdir('newchild')
+        open('b', 'wb').write('newchild b contents\n')
+        bzr('commit', '-m', 'newchild')
+        self.check_revno(2)
+
+        os.chdir('../child')
+        # The pull should succeed, and update
+        # the bound parent branch
+        bzr('pull', '../newchild')
+        self.check_revno(2)
+
+        self.check_revno(2, '../base')
+
+    def test_bind_diverged(self):
+        bzr = self.run_bzr
+        self.create_branches()
+
+        os.chdir('child')
+        bzr('unbind')
+
+        bzr('commit', '-m', 'child', '--unchanged')
+        self.check_revno(2)
+
+        os.chdir('../base')
+        self.check_revno(1)
+        bzr('commit', '-m', 'base', '--unchanged')
+        self.check_revno(2)
+
+        os.chdir('../child')
+        # These branches have diverged
+        bzr('bind', '../base', retcode=3)
+
+        # TODO: In the future, this might require actual changes
+        # to have occurred, rather than just a new revision entry
+        bzr('merge', '../base')
+        bzr('commit', '-m', 'merged')
+        self.check_revno(3)
+
+        # After a merge, trying to bind again should succeed
+        # by pushing the new change to base
+        bzr('bind', '../base')
+        self.check_revno(3)
+        self.check_revno(3, '../base')
+
+        # After binding, the revision history should be identical
+        child_rh = bzr('revision-history')[0]
+        os.chdir('../base')
+        base_rh = bzr('revision-history')[0]
+        self.assertEquals(child_rh, base_rh)
+
+    def test_bind_parent_ahead(self):
+        bzr = self.run_bzr
+        self.create_branches()
+
+        os.chdir('child')
+        bzr('unbind')
+
+        os.chdir('../base')
+        bzr('commit', '-m', 'base', '--unchanged')
+
+        os.chdir('../child')
+        self.check_revno(1)
+        bzr('bind', '../base')
+
+        self.check_revno(2)
+        bzr('unbind')
+
+        # Check and make sure it also works if parent is ahead multiple
+        os.chdir('../base')
+        bzr('commit', '-m', 'base 3', '--unchanged')
+        bzr('commit', '-m', 'base 4', '--unchanged')
+        bzr('commit', '-m', 'base 5', '--unchanged')
+        self.check_revno(5)
+
+        os.chdir('../child')
+        self.check_revno(2)
+        bzr('bind', '../base')
+        self.check_revno(5)
+
+    def test_bind_child_ahead(self):
+        bzr = self.run_bzr
+        self.create_branches()
+
+        os.chdir('child')
+        bzr('unbind')
+        bzr('commit', '-m', 'child', '--unchanged')
+        self.check_revno(2)
+        self.check_revno(1, '../base')
+
+        bzr('bind', '../base')
+        self.check_revno(2, '../base')
+
+        # Check and make sure it also works if child is ahead multiple
+        bzr('unbind')
+        bzr('commit', '-m', 'child 3', '--unchanged')
+        bzr('commit', '-m', 'child 4', '--unchanged')
+        bzr('commit', '-m', 'child 5', '--unchanged')
+        self.check_revno(5)
+
+        self.check_revno(2, '../base')
+        bzr('bind', '../base')
+        self.check_revno(5, '../base')
+
+    def test_commit_after_merge(self):
+        bzr = self.run_bzr
+        self.create_branches()
+
+        # We want merge to be able to be a local only
+        # operation, because it can be without violating
+        # the binding invariants.
+        # But we can't fail afterwards
+
+        bzr('branch', 'child', 'other')
+
+        os.chdir('other')
+        open('c', 'wb').write('file c\n')
+        bzr('add', 'c')
+        bzr('commit', '-m', 'adding c')
+        new_rev_id = bzr('revision-history')[0].strip().split('\n')[-1]
+
+        os.chdir('../child')
+        bzr('merge', '../other')
+
+        self.failUnlessExists('c')
+        tree = WorkingTree.open('.')
+        self.assertEqual([new_rev_id], tree.pending_merges())
+
+        # Make sure the local branch has the installed revision
+        bzr('cat-revision', new_rev_id)
+        
+        # And make sure that the base tree does not
+        os.chdir('../base')
+        bzr('cat-revision', new_rev_id, retcode=3)
+
+        # Commit should succeed, and cause merged revisions to
+        # be pulled into base
+        os.chdir('../child')
+        bzr('commit', '-m', 'merge other')
+
+        self.check_revno(2)
+
+        os.chdir('../base')
+        self.check_revno(2)
+
+        bzr('cat-revision', new_rev_id)
+
+    def test_pull_overwrite_fails(self):
+        bzr = self.run_bzr
+        self.create_branches()
+
+        bzr('branch', 'child', 'other')
+        
+        os.chdir('other')
+        open('a', 'wb').write('new contents\n')
+        bzr('commit', '-m', 'changed a')
+        self.check_revno(2)
+        open('a', 'ab').write('and then some\n')
+        bzr('commit', '-m', 'another a')
+        self.check_revno(3)
+        open('a', 'ab').write('and some more\n')
+        bzr('commit', '-m', 'yet another a')
+        self.check_revno(4)
+
+        os.chdir('../child')
+        open('a', 'wb').write('also changed a\n')
+        bzr('commit', '-m', 'child modified a')
+
+        self.check_revno(2)
+        self.check_revno(2, '../base')
+
+        # It might be possible that we want pull --overwrite to
+        # actually succeed.
+        # If we want it, just change this test to make sure that 
+        # both base and child are updated properly
+        bzr('pull', '--overwrite', '../other', retcode=3)
+
+        # It should fail without changing the local revision
+        self.check_revno(2)
+        self.check_revno(2, '../base')
+
+    # TODO: jam 20051230 Test that commit & pull fail when the branch we 
+    #       are bound to is not available
+
+

=== modified file 'bzrlib/tests/blackbox/test_checkout.py'
--- bzrlib/tests/blackbox/test_checkout.py	2006-02-12 09:40:15 +0000
+++ bzrlib/tests/blackbox/test_checkout.py	2006-03-05 11:48:54 +0000
@@ -37,12 +37,20 @@
         tree.add('added_in_2')
         tree.commit('2', rev_id='2')
 
-    def test_checkout_makes_checkout(self):
+    def test_checkout_makes_bound_branch(self):
         self.runbzr('checkout branch checkout')
         # if we have a checkout, the branch base should be 'branch'
         source = bzrdir.BzrDir.open('branch')
         result = bzrdir.BzrDir.open('checkout')
         self.assertEqual(source.open_branch().bzrdir.root_transport.base,
+                         result.open_branch().get_bound_location())
+
+    def test_checkout_light_makes_checkout(self):
+        self.runbzr('checkout --lightweight branch checkout')
+        # if we have a checkout, the branch base should be 'branch'
+        source = bzrdir.BzrDir.open('branch')
+        result = bzrdir.BzrDir.open('checkout')
+        self.assertEqual(source.open_branch().bzrdir.root_transport.base,
                          result.open_branch().bzrdir.root_transport.base)
 
     def test_checkout_dash_r(self):
@@ -53,3 +61,11 @@
         self.assertEqual('1', result.open_workingtree().last_revision())
         self.failIfExists('checkout/added_in_2')
 
+    def test_checkout_light_dash_r(self):
+        self.runbzr('checkout --lightweight -r -2 branch checkout')
+        # the working tree should now be at revision '1' with the content
+        # from 1.
+        result = bzrdir.BzrDir.open('checkout')
+        self.assertEqual('1', result.open_workingtree().last_revision())
+        self.failIfExists('checkout/added_in_2')
+

=== modified file 'bzrlib/tests/blackbox/test_commit.py'
--- bzrlib/tests/blackbox/test_commit.py	2006-02-12 10:19:13 +0000
+++ bzrlib/tests/blackbox/test_commit.py	2006-03-05 11:48:54 +0000
@@ -71,7 +71,7 @@
         # of date checkout
         self.make_branch_and_tree('branch')
         # make a checkout
-        self.runbzr('checkout branch checkout')
+        self.runbzr('checkout --lightweight branch checkout')
         # commit to the original branch to make the checkout out of date
         self.runbzr('commit --unchanged -m message branch')
         # now commit to the checkout should emit
@@ -82,3 +82,11 @@
                          ('',
                           "bzr: ERROR: Working tree is out of date, please run "
                           "'bzr update'.\n"))
+
+    def test_local_commit_unbound(self):
+        # a --local commit on an unbound branch is an error
+        self.make_branch_and_tree('.')
+        out, err = self.run_bzr('commit', '--local', retcode=3)
+        self.assertEqualDiff('', out)
+        self.assertEqualDiff('bzr: ERROR: Cannot perform local-only commits '
+                             'on unbound branches.\n', err)

=== modified file 'bzrlib/tests/blackbox/test_info.py'
--- bzrlib/tests/blackbox/test_info.py	2006-02-15 06:08:08 +0000
+++ bzrlib/tests/blackbox/test_info.py	2006-03-05 11:48:54 +0000
@@ -58,6 +58,35 @@
         self.runbzr('checkout branch checkout')
         out, err = self.runbzr('info checkout')
         self.assertEqualDiff(
+"""branch format: Bazaar-NG Metadir branch format 5
+bound to branch: %s
+
+in the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+branch history:
+         0 revisions
+         0 committers
+
+revision store:
+         0 revisions
+         0 kB
+""" % a_branch.bzrdir.root_transport.base,
+        out)
+        self.assertEqual('', err)
+
+    def test_info_up_to_date_light_checkout(self):
+        a_branch = self.make_branch_and_tree('branch')
+        self.runbzr('checkout --lightweight branch checkout')
+        out, err = self.runbzr('info checkout')
+        self.assertEqualDiff(
 """working tree format: Bazaar-NG Working Tree format 3
 branch location: %s
 branch format: Bazaar-NG branch, format 6
@@ -136,6 +165,48 @@
         rev = a_branch.repository.get_revision(a_branch.revision_history()[0])
         datestring = format_date(rev.timestamp, rev.timezone)
         self.assertEqualDiff(
+"""branch format: Bazaar-NG Metadir branch format 5
+bound to branch: %s
+
+Branch is out of date: missing 1 revision.
+in the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+branch history:
+         0 revisions
+         0 committers
+
+revision store:
+         0 revisions
+         0 kB
+""" % (a_branch.bzrdir.root_transport.base,
+       ),
+            out)
+        self.assertEqual('', err)
+
+    def test_info_out_of_date_light_checkout(self):
+        # note this deliberately uses a checkout at 'None' to 
+        # test the out of date message with a revision notin the 
+        # revision history.
+        a_branch = self.make_branch('branch')
+        # make two checkouts
+        self.runbzr('checkout --lightweight branch checkout')
+        self.runbzr('checkout --lightweight branch checkout2')
+        self.build_tree(['checkout/file'])
+        self.runbzr('add checkout/file')
+        self.runbzr('commit -m add-file checkout')
+        # now checkout2 should be out of date
+        out,err = self.runbzr('info checkout2')
+        rev = a_branch.repository.get_revision(a_branch.revision_history()[0])
+        datestring = format_date(rev.timestamp, rev.timezone)
+        self.assertEqualDiff(
 """working tree format: Bazaar-NG Working Tree format 3
 branch location: %s
 branch format: Bazaar-NG branch, format 6

=== modified file 'bzrlib/tests/blackbox/test_update.py'
--- bzrlib/tests/blackbox/test_update.py	2006-02-21 01:05:33 +0000
+++ bzrlib/tests/blackbox/test_update.py	2006-03-05 11:48:54 +0000
@@ -21,6 +21,7 @@
 
 from bzrlib.tests import TestSkipped
 from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.workingtree import WorkingTree
 
 
 class TestUpdate(ExternalBase):
@@ -31,9 +32,9 @@
         self.assertEqual('Tree is up to date.\n', err)
         self.assertEqual('', out)
 
-    def test_update_up_to_date_checkout(self):
+    def test_update_up_to_date_light_checkout(self):
         self.make_branch_and_tree('branch')
-        self.runbzr('checkout branch checkout')
+        self.runbzr('checkout --lightweight branch checkout')
         out, err = self.runbzr('update checkout')
         self.assertEqual('Tree is up to date.\n', err)
         self.assertEqual('', out)
@@ -44,7 +45,7 @@
         raise TestSkipped('default format too old')
         self.make_branch_and_tree('branch')
         # make a checkout
-        self.runbzr('checkout branch checkout')
+        self.runbzr('checkout --lightweight branch checkout')
         self.build_tree(['checkout/file'])
         self.runbzr('add checkout/file')
         self.runbzr('commit -m add-file checkout')
@@ -54,11 +55,11 @@
         self.assertEqual('', err)
         self.failUnlessExists('branch/file')
 
-    def test_update_out_of_date_checkout(self):
+    def test_update_out_of_date_light_checkout(self):
         self.make_branch_and_tree('branch')
         # make two checkouts
-        self.runbzr('checkout branch checkout')
-        self.runbzr('checkout branch checkout2')
+        self.runbzr('checkout --lightweight branch checkout')
+        self.runbzr('checkout --lightweight branch checkout2')
         self.build_tree(['checkout/file'])
         self.runbzr('add checkout/file')
         self.runbzr('commit -m add-file checkout')
@@ -72,11 +73,11 @@
     def test_update_conflicts_returns_2(self):
         self.make_branch_and_tree('branch')
         # make two checkouts
-        self.runbzr('checkout branch checkout')
+        self.runbzr('checkout --lightweight branch checkout')
         self.build_tree(['checkout/file'])
         self.runbzr('add checkout/file')
         self.runbzr('commit -m add-file checkout')
-        self.runbzr('checkout branch checkout2')
+        self.runbzr('checkout --lightweight branch checkout2')
         # now alter file in checkout
         a_file = file('checkout/file', 'wt')
         a_file.write('Foo')
@@ -93,3 +94,39 @@
                          err.split('\n')[1:3])
         self.assertContainsRe(err, 'Text conflict in file\n')
         self.assertEqual('', out)
+
+    def test_smoke_update_checkout_bound_branch_local_commits(self):
+        # smoke test for doing an update of a checkout of a bound
+        # branch with local commits.
+        self.make_branch_and_tree('master')
+        # make a bound branch
+        self.run_bzr('checkout', 'master', 'child')
+        # check that out
+        self.run_bzr('checkout', '--lightweight', 'child', 'checkout')
+        # change master
+        a_file = file('master/file', 'wt')
+        a_file.write('Foo')
+        a_file.close()
+        self.run_bzr('add', 'master')
+        self.run_bzr('commit', '-m', 'add file', 'master')
+        # change child
+        a_file = file('child/file_b', 'wt')
+        a_file.write('Foo')
+        a_file.close()
+        self.run_bzr('add', 'child')
+        self.run_bzr('commit', '--local', '-m', 'add file_b', 'child')
+        # check checkout
+        a_file = file('checkout/file_c', 'wt')
+        a_file.write('Foo')
+        a_file.close()
+        self.run_bzr('add', 'checkout')
+
+        # now, update checkout ->
+        # get all three files and a pending merge.
+        self.run_bzr('update', 'checkout')
+        wt = WorkingTree.open('checkout')
+        self.assertNotEqual([], wt.pending_merges())
+        self.failUnlessExists('checkout/file')
+        self.failUnlessExists('checkout/file_b')
+        self.failUnlessExists('checkout/file_c')
+        self.assertTrue(wt.has_filename('file_c'))

=== modified file 'bzrlib/tests/blackbox/test_upgrade.py'
--- bzrlib/tests/blackbox/test_upgrade.py	2006-02-23 04:08:56 +0000
+++ bzrlib/tests/blackbox/test_upgrade.py	2006-03-05 11:48:54 +0000
@@ -49,6 +49,7 @@
         d.create_branch()
         d.create_workingtree()
         self.run_bzr('checkout',
+                     '--lightweight',
                      self.get_url('current_format_branch'),
                      'current_format_checkout')
 

=== modified file 'bzrlib/tests/branch_implementations/__init__.py'
--- bzrlib/tests/branch_implementations/__init__.py	2006-02-10 11:03:38 +0000
+++ bzrlib/tests/branch_implementations/__init__.py	2006-03-05 00:19:31 +0000
@@ -39,9 +39,11 @@
 def test_suite():
     result = TestSuite()
     test_branch_implementations = [
+        'bzrlib.tests.branch_implementations.test_bound_sftp',
         'bzrlib.tests.branch_implementations.test_branch',
         'bzrlib.tests.branch_implementations.test_parent',
         'bzrlib.tests.branch_implementations.test_permissions',
+        'bzrlib.tests.branch_implementations.test_update',
         ]
     adapter = BranchTestProviderAdapter(
         default_transport,

=== added file 'bzrlib/tests/branch_implementations/test_bound_sftp.py'
--- bzrlib/tests/branch_implementations/test_bound_sftp.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/branch_implementations/test_bound_sftp.py	2006-03-04 13:03:57 +0000
@@ -0,0 +1,366 @@
+# Copyright (C) 2005 Robey Pointer <robey@lag.net>, Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for branches bound to an sftp branch."""
+
+
+import os
+
+
+from bzrlib.branch import Branch
+from bzrlib.bzrdir import (BzrDir,
+                           BzrDirFormat,
+                           BzrDirFormat6,
+                           BzrDirMetaFormat1,
+                           )
+import bzrlib.errors as errors
+from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer, paramiko_loaded
+
+
+class BoundSFTPBranch(TestCaseWithSFTPServer):
+
+    def create_branches(self):
+        self.build_tree(['base/', 'base/a', 'base/b'])
+        old_format = BzrDirFormat.get_default_format()
+        BzrDirFormat.set_default_format(BzrDirMetaFormat1())
+        try:
+            wt_base = BzrDir.create_standalone_workingtree('base')
+        finally:
+            BzrDirFormat.set_default_format(old_format)
+    
+        b_base = wt_base.branch
+
+        wt_base.add('a')
+        wt_base.add('b')
+        wt_base.commit('first', rev_id='r@b-1')
+
+        wt_child = b_base.bzrdir.sprout('child').open_workingtree()
+        self.sftp_base = Branch.open(self.get_url('base'))
+        wt_child.branch.bind(self.sftp_base)
+
+        self.assertEqual(['r@b-1'], b_base.revision_history())
+        self.assertEqual(['r@b-1'], wt_child.branch.revision_history())
+
+        return b_base, wt_child
+
+    def test_simple_binding(self):
+        self.build_tree(['base/', 'base/a', 'base/b', 'child/'])
+        wt_base = BzrDir.create_standalone_workingtree('base')
+
+        wt_base.add('a')
+        wt_base.add('b')
+        wt_base.commit('first', rev_id='r@b-1')
+
+        b_base = wt_base.branch
+        old_format = BzrDirFormat.get_default_format()
+        BzrDirFormat.set_default_format(BzrDirMetaFormat1())
+        try:
+            b_child = BzrDir.create_branch_convenience('child')
+        finally:
+            BzrDirFormat.set_default_format(old_format)
+        self.assertEqual(None, b_child.get_bound_location())
+        self.assertEqual(None, b_child.get_master_branch())
+
+        sftp_b_base = Branch.open(self.get_url('base'))
+        b_child.bind(sftp_b_base)
+        self.assertEqual(sftp_b_base.base, b_child.get_bound_location())
+        # this line is more of a working tree test line, but - what the hey.
+        b_child.bzrdir.open_workingtree().update()
+        self.failUnlessExists('child/a')
+        self.failUnlessExists('child/b')
+
+        b_child.unbind()
+        self.assertEqual(None, b_child.get_bound_location())
+
+    def test_bound_commit(self):
+        b_base, wt_child = self.create_branches()
+
+        open('child/a', 'wb').write('new contents\n')
+        wt_child.commit('modified a', rev_id='r@c-2')
+
+        self.assertEqual(['r@b-1', 'r@c-2'], wt_child.branch.revision_history())
+        self.assertEqual(['r@b-1', 'r@c-2'], b_base.revision_history())
+
+    def test_bound_fail(self):
+        # Make sure commit fails if out of date.
+        b_base, wt_child = self.create_branches()
+
+        open('base/a', 'wb').write('new base contents\n')
+        b_base.bzrdir.open_workingtree().commit('base', rev_id='r@b-2')
+
+        open('child/b', 'wb').write('new b child contents\n')
+        self.assertRaises(errors.BoundBranchOutOfDate,
+                wt_child.commit, 'child', rev_id='r@c-2')
+
+        sftp_b_base = Branch.open(self.get_url('base'))
+
+        # This is all that cmd_update does
+        wt_child.pull(sftp_b_base, overwrite=False)
+
+        wt_child.commit('child', rev_id='r@c-3')
+
+        self.assertEqual(['r@b-1', 'r@b-2', 'r@c-3'],
+                wt_child.branch.revision_history())
+        self.assertEqual(['r@b-1', 'r@b-2', 'r@c-3'],
+                b_base.revision_history())
+        self.assertEqual(['r@b-1', 'r@b-2', 'r@c-3'],
+                sftp_b_base.revision_history())
+
+    def test_double_binding(self):
+        b_base, wt_child = self.create_branches()
+
+        wt_child2 = wt_child.bzrdir.sprout('child2').open_workingtree()
+
+        wt_child2.branch.bind(wt_child.branch)
+
+        open('child2/a', 'wb').write('new contents\n')
+        self.assertRaises(errors.CommitToDoubleBoundBranch,
+                wt_child2.commit, 'child2', rev_id='r@d-2')
+
+    def test_unbinding(self):
+        from bzrlib.transport import get_transport
+        b_base, wt_child = self.create_branches()
+
+        # TestCaseWithSFTPServer only allows you to connect one time
+        # to the SFTP server. So we have to create a connection and
+        # keep it around, so that it can be reused
+        __unused_t = get_transport(self.get_url('.'))
+
+        wt_base = b_base.bzrdir.open_workingtree()
+        open('base/a', 'wb').write('new base contents\n')
+        wt_base.commit('base', rev_id='r@b-2')
+
+        open('child/b', 'wb').write('new b child contents\n')
+        self.assertRaises(errors.BoundBranchOutOfDate,
+                wt_child.commit, 'child', rev_id='r@c-2')
+        self.assertEqual(['r@b-1'], wt_child.branch.revision_history())
+        wt_child.branch.unbind()
+        wt_child.commit('child', rev_id='r@c-2')
+        self.assertEqual(['r@b-1', 'r@c-2'], wt_child.branch.revision_history())
+        self.assertEqual(['r@b-1', 'r@b-2'], b_base.revision_history())
+
+        sftp_b_base = Branch.open(self.get_url('base'))
+        self.assertRaises(errors.DivergedBranches,
+                wt_child.branch.bind, sftp_b_base)
+
+    def test_commit_remote_bound(self):
+        # Make sure it is detected if the current base
+        # suddenly is bound when child goes to commit
+        b_base, wt_child = self.create_branches()
+
+        b_base.bzrdir.sprout('newbase')
+
+        sftp_b_base = Branch.open(self.get_url('base'))
+        sftp_b_newbase = Branch.open(self.get_url('newbase'))
+
+        sftp_b_base.bind(sftp_b_newbase)
+
+        open('child/a', 'wb').write('new contents\n')
+        self.assertRaises(errors.CommitToDoubleBoundBranch,
+                wt_child.commit, 'failure', rev_id='r@c-2')
+
+        self.assertEqual(['r@b-1'], b_base.revision_history())
+        self.assertEqual(['r@b-1'], wt_child.branch.revision_history())
+        self.assertEqual(['r@b-1'], sftp_b_newbase.revision_history())
+
+    def test_pull_updates_both(self):
+        b_base, wt_child = self.create_branches()
+
+        wt_newchild = b_base.bzrdir.sprout('newchild').open_workingtree()
+        open('newchild/b', 'wb').write('newchild b contents\n')
+        wt_newchild.commit('newchild', rev_id='r@d-2')
+        self.assertEqual(['r@b-1', 'r@d-2'], wt_newchild.branch.revision_history())
+
+        wt_child.pull(wt_newchild.branch)
+        self.assertEqual(['r@b-1', 'r@d-2'], wt_child.branch.revision_history())
+        self.assertEqual(['r@b-1', 'r@d-2'], b_base.revision_history())
+
+    def test_bind_diverged(self):
+        from bzrlib.builtins import merge
+
+        b_base, wt_child = self.create_branches()
+
+        wt_child.branch.unbind()
+        open('child/a', 'ab').write('child contents\n')
+        wt_child.commit('child', rev_id='r@c-2')
+
+        self.assertEqual(['r@b-1', 'r@c-2'], wt_child.branch.revision_history())
+        self.assertEqual(['r@b-1'], b_base.revision_history())
+
+        open('base/b', 'ab').write('base contents\n')
+        b_base.bzrdir.open_workingtree().commit('base', rev_id='r@b-2')
+        self.assertEqual(['r@b-1', 'r@b-2'], b_base.revision_history())
+
+        sftp_b_base = Branch.open(self.get_url('base'))
+
+        self.assertRaises(errors.DivergedBranches,
+                wt_child.branch.bind, sftp_b_base)
+
+        # TODO: jam 20051230 merge_inner doesn't set pending merges
+        #       Is this on purpose?
+        #       merge_inner also doesn't fetch any missing revisions
+        #merge_inner(wt_child.branch, sftp_b_base.revision_tree('r@b-2'), 
+        #        wt_child.branch.revision_tree('r@b-1'))
+        # TODO: jam 20051230 merge(..., (None, None), ...) seems to
+        #       cause an infinite loop of some sort. It definitely doesn't
+        #       work, you have to use list notation
+        merge((sftp_b_base.base, 2), [None, None], this_dir=wt_child.branch.base)
+
+        self.assertEqual(['r@b-2'], wt_child.pending_merges())
+        wt_child.commit('merged', rev_id='r@c-3')
+
+        # After a merge, trying to bind again should succeed
+        # by pushing the new change to base
+        wt_child.branch.bind(sftp_b_base)
+
+        self.assertEqual(['r@b-1', 'r@b-2', 'r@c-3'],
+                b_base.revision_history())
+        self.assertEqual(['r@b-1', 'r@b-2', 'r@c-3'],
+                wt_child.branch.revision_history())
+
+    def test_bind_parent_ahead(self):
+        b_base, wt_child = self.create_branches()
+
+        wt_child.branch.unbind()
+
+        open('a', 'ab').write('base changes\n')
+        wt_base = b_base.bzrdir.open_workingtree()
+        wt_base.commit('base', rev_id='r@b-2')
+        self.assertEqual(['r@b-1', 'r@b-2'], b_base.revision_history())
+        self.assertEqual(['r@b-1'], wt_child.branch.revision_history())
+
+        sftp_b_base = Branch.open(self.get_url('base'))
+        wt_child.branch.bind(sftp_b_base)
+
+        self.assertEqual(['r@b-1', 'r@b-2'], wt_child.branch.revision_history())
+
+        wt_child.branch.unbind()
+
+        # Check and make sure it also works if parent is ahead multiple
+        wt_base.commit('base 3', rev_id='r@b-3', allow_pointless=True)
+        wt_base.commit('base 4', rev_id='r@b-4', allow_pointless=True)
+        wt_base.commit('base 5', rev_id='r@b-5', allow_pointless=True)
+
+        self.assertEqual(['r@b-1', 'r@b-2', 'r@b-3', 'r@b-4', 'r@b-5'],
+                b_base.revision_history())
+
+        self.assertEqual(['r@b-1', 'r@b-2'], wt_child.branch.revision_history())
+
+        wt_child.branch.bind(sftp_b_base)
+        self.assertEqual(['r@b-1', 'r@b-2', 'r@b-3', 'r@b-4', 'r@b-5'],
+                wt_child.branch.revision_history())
+
+    def test_bind_child_ahead(self):
+        b_base, wt_child = self.create_branches()
+
+        wt_child.branch.unbind()
+
+        wt_child.commit('child', rev_id='r@c-2', allow_pointless=True)
+        self.assertEqual(['r@b-1', 'r@c-2'], wt_child.branch.revision_history())
+        self.assertEqual(['r@b-1'], b_base.revision_history())
+
+        sftp_b_base = Branch.open(self.get_url('base'))
+        wt_child.branch.bind(sftp_b_base)
+
+        self.assertEqual(['r@b-1', 'r@c-2'], b_base.revision_history())
+
+        # Check and make sure it also works if child is ahead multiple
+        wt_child.branch.unbind()
+        wt_child.commit('child 3', rev_id='r@c-3', allow_pointless=True)
+        wt_child.commit('child 4', rev_id='r@c-4', allow_pointless=True)
+        wt_child.commit('child 5', rev_id='r@c-5', allow_pointless=True)
+
+        self.assertEqual(['r@b-1', 'r@c-2', 'r@c-3', 'r@c-4', 'r@c-5'],
+                wt_child.branch.revision_history())
+        self.assertEqual(['r@b-1', 'r@c-2'], b_base.revision_history())
+
+        wt_child.branch.bind(sftp_b_base)
+        self.assertEqual(['r@b-1', 'r@c-2', 'r@c-3', 'r@c-4', 'r@c-5'],
+                b_base.revision_history())
+
+    def test_commit_after_merge(self):
+        from bzrlib.builtins import merge
+
+        b_base, wt_child = self.create_branches()
+
+        # We want merge to be able to be a local only
+        # operation, because it does not alter the branch data.
+
+        # But we can't fail afterwards
+
+        wt_other = wt_child.bzrdir.sprout('other').open_workingtree()
+
+        open('other/c', 'wb').write('file c\n')
+        wt_other.add('c')
+        wt_other.commit('adding c', rev_id='r@d-2')
+
+        self.failIf(wt_child.branch.repository.has_revision('r@d-2'))
+        self.failIf(b_base.repository.has_revision('r@d-2'))
+
+        # TODO: jam 20051230 merge_inner doesn't set pending merges
+        #       Is this on purpose?
+        #       merge_inner also doesn't fetch any missing revisions
+        #merge_inner(wt_child.branch, b_other.revision_tree('r@d-2'),
+        #        wt_child.branch.revision_tree('r@b-1'))
+        merge((wt_other.branch.base, 2), [None, None], this_dir=wt_child.branch.base)
+
+        self.failUnlessExists('child/c')
+        self.assertEqual(['r@d-2'], wt_child.pending_merges())
+        self.failUnless(wt_child.branch.repository.has_revision('r@d-2'))
+        self.failIf(b_base.repository.has_revision('r@d-2'))
+
+        # Commit should succeed, and cause merged revisions to
+        # be pulled into base
+        wt_child.commit('merge other', rev_id='r@c-2')
+        self.assertEqual(['r@b-1', 'r@c-2'], wt_child.branch.revision_history())
+        self.assertEqual(['r@b-1', 'r@c-2'], b_base.revision_history())
+        self.failUnless(b_base.repository.has_revision('r@d-2'))
+
+    def test_commit_fails(self):
+        b_base, wt_child = self.create_branches()
+
+        open('a', 'ab').write('child adds some text\n')
+
+        del b_base
+        os.rename('base', 'hidden_base')
+
+        self.assertRaises(errors.BoundBranchConnectionFailure,
+                wt_child.commit, 'added text', rev_id='r@c-2')
+
+    def test_pull_fails(self):
+        b_base, wt_child = self.create_branches()
+
+        wt_other = wt_child.bzrdir.sprout('other').open_workingtree()
+        open('other/a', 'wb').write('new contents\n')
+        wt_other.commit('changed a', rev_id='r@d-2')
+
+        self.assertEqual(['r@b-1'], b_base.revision_history())
+        self.assertEqual(['r@b-1'], wt_child.branch.revision_history())
+        self.assertEqual(['r@b-1', 'r@d-2'], wt_other.branch.revision_history())
+
+        del b_base
+        os.rename('base', 'hidden_base')
+
+        self.assertRaises(errors.BoundBranchConnectionFailure,
+                wt_child.pull, wt_other.branch)
+
+    # TODO: jam 20051231 We need invasive failure tests, so that we can show
+    #       performance even when something fails.
+
+
+if not paramiko_loaded:
+    del BoundSFTPBranch
+

=== added file 'bzrlib/tests/branch_implementations/test_update.py'
--- bzrlib/tests/branch_implementations/test_update.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/branch_implementations/test_update.py	2006-03-05 02:20:07 +0000
@@ -0,0 +1,65 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import bzrlib.errors as errors
+from bzrlib.tests.branch_implementations.test_branch import TestCaseWithBranch
+
+
+"""Tests for branch.update()"""
+
+
+class TestUpdate(TestCaseWithBranch):
+
+    def test_update_unbound_works(self):
+        b = self.make_branch('.')
+        b.update()
+        self.assertEqual(None, b.last_revision())
+
+    def test_update_prefix_returns_none(self):
+        # update in a branch when its a prefix of the master should
+        # indicate that no local changes were present.
+        master_tree = self.make_branch_and_tree('master')
+        child_tree = self.make_branch_and_tree('child')
+        try:
+            child_tree.branch.bind(master_tree.branch)
+        except errors.UpgradeRequired:
+            # old branch, cant test.
+            return
+        # commit to the child to make the last rev not-None.
+        child_tree.commit('foo', rev_id='foo', allow_pointless=True)
+        # update the master so we can commit there.
+        master_tree.update()
+        # commit to the master making the child tree out of date and a prefix.
+        master_tree.commit('bar', rev_id='bar', allow_pointless=True)
+        self.assertEqual(None, child_tree.branch.update())
+
+    def test_update_local_commits_returns_old_tip(self):
+        # update in a branch when its not a prefix of the master should
+        # return the previous tip and reset the revision history.
+        master_tree = self.make_branch_and_tree('master')
+        child_tree = self.make_branch_and_tree('child')
+        try:
+            child_tree.branch.bind(master_tree.branch)
+        except errors.UpgradeRequired:
+            # old branch, cant test.
+            return
+        # commit to the child to make the last rev not-None and skew it from master.
+        child_tree.commit('foo', rev_id='foo', allow_pointless=True, local=True)
+        # commit to the master making the child tree out of date and not a prefix.
+        master_tree.commit('bar', rev_id='bar', allow_pointless=True)
+        self.assertEqual('foo', child_tree.branch.update())
+        self.assertEqual(['bar'], child_tree.branch.revision_history())

=== modified file 'bzrlib/tests/bzrdir_implementations/test_bzrdir.py'
--- bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2006-02-22 05:58:46 +0000
+++ bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2006-03-03 13:22:46 +0000
@@ -499,7 +499,12 @@
         dir = source.bzrdir
         target = dir.sprout(self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/stat-cache',
+                                     './.bzr/checkout/stat-cache',
+                                     './.bzr/inventory',
+                                     './.bzr/checkout/inventory',
+                                     ])
 
     def test_sprout_bzrdir_branch_and_repo_shared(self):
         # sprouting a branch with a repo into a shared repo uses the shared
@@ -634,11 +639,15 @@
         target = dir.sprout(self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    ['./.bzr/stat-cache', './.bzr/checkout/stat-cache'])
+                                    ['./.bzr/stat-cache',
+                                     './.bzr/checkout/stat-cache',
+                                     './.bzr/inventory',
+                                     './.bzr/checkout/inventory',
+                                     ])
 
     def test_sprout_bzrdir_tree_branch_reference(self):
         # sprouting should create a repository if needed and a sprouted branch.
-        # the tree state should be copied.
+        # the tree state should not be copied.
         referenced_branch = self.make_branch('referencced')
         dir = self.make_bzrdir('source')
         try:
@@ -648,7 +657,9 @@
             # this is ok too, not all formats have to support references.
             return
         self.assertRaises(errors.NoRepositoryPresent, dir.open_repository)
-        dir.create_workingtree()
+        tree = dir.create_workingtree()
+        tree.bzrdir.root_transport.mkdir('subdir')
+        tree.add('subdir')
         target = dir.sprout(self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         # we want target to have a branch that is in-place.
@@ -656,12 +667,12 @@
         # and as we dont support repositories being detached yet, a repo in 
         # place
         target.open_repository()
-        # we trust that the working tree sprouting works via the other tests.
-        target.open_workingtree()
+        result_tree = target.open_workingtree()
+        self.assertFalse(result_tree.has_filename('subdir'))
 
     def test_sprout_bzrdir_tree_branch_reference_revision(self):
         # sprouting should create a repository if needed and a sprouted branch.
-        # the tree state should be copied but the revision changed,
+        # the tree state should not be copied but the revision changed,
         # and the likewise the new branch should be truncated too
         referenced_branch = self.make_branch('referencced')
         dir = self.make_bzrdir('source')

=== modified file 'bzrlib/tests/test_revision.py'
--- bzrlib/tests/test_revision.py	2006-03-03 09:41:00 +0000
+++ bzrlib/tests/test_revision.py	2006-03-05 02:20:07 +0000
@@ -246,6 +246,7 @@
                           revisions[4]) # revisions_2[5] is equally valid
         self.assertEqual(common_ancestor(revisions_2[6], revisions[5], sources),
                           revisions[4]) # revisions_2[5] is equally valid
+        self.assertEqual(None, common_ancestor(None, revisions[5], sources))
 
     def test_combined(self):
         """combined_graph

=== modified file 'bzrlib/tests/test_source.py'
--- bzrlib/tests/test_source.py	2006-02-11 11:58:06 +0000
+++ bzrlib/tests/test_source.py	2006-03-05 02:20:07 +0000
@@ -72,4 +72,4 @@
         # written. Note that this is an exact equality so that when the number
         # drops, it is not given a buffer but rather this test updated
         # immediately.
-        self.assertEqual(6, occurences)
+        self.assertEqual(4, occurences)

=== modified file 'bzrlib/tests/workingtree_implementations/test_workingtree.py'
--- bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-02-22 05:45:47 +0000
+++ bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-03-05 08:31:28 +0000
@@ -327,6 +327,50 @@
         tree.commit('foo', rev_id='foo', allow_pointless=True)
         self.assertEqual('foo', tree.last_revision())
 
+    def test_commit_local_unbound(self):
+        # using the library api to do a local commit on unbound branches is 
+        # also an error
+        tree = self.make_branch_and_tree('tree')
+        self.assertRaises(errors.LocalRequiresBoundBranch,
+                          tree.commit,
+                          'foo',
+                          local=True)
+ 
+    def test_local_commit_ignores_master(self):
+        # a --local commit does not require access to the master branch
+        # at all, or even for it to exist.
+        # we test this by setting up a bound branch and then corrupting
+        # the master.
+        master = self.make_branch('master')
+        tree = self.make_branch_and_tree('tree')
+        try:
+            tree.branch.bind(master)
+        except errors.UpgradeRequired:
+            # older format.
+            return
+        master.bzrdir.transport.put('branch-format', StringIO('garbage'))
+        del master
+        # check its corrupted.
+        self.assertRaises(errors.UnknownFormatError,
+                          bzrdir.BzrDir.open,
+                          'master')
+        tree.commit('foo', rev_id='foo', local=True)
+ 
+    def test_local_commit_does_not_push_to_master(self):
+        # a --local commit does not require access to the master branch
+        # at all, or even for it to exist.
+        # we test that even when its available it does not push to it.
+        master = self.make_branch('master')
+        tree = self.make_branch_and_tree('tree')
+        try:
+            tree.branch.bind(master)
+        except errors.UpgradeRequired:
+            # older format.
+            return
+        tree.commit('foo', rev_id='foo', local=True)
+        self.failIf(master.repository.has_revision('foo'))
+        self.assertEqual(None, master.last_revision())
+        
     def test_update_sets_last_revision(self):
         # working tree formats from the meta-dir format and newer support
         # setting the last revision on a tree independently of that on the 
@@ -391,3 +435,41 @@
         self.assertEqual(1, old_tree.update())
         self.assertEqual('A', old_tree.last_revision())
 
+    def test_update_updates_bound_branch_no_local_commits(self):
+        # doing an update in a tree updates the branch its bound to too.
+        master_tree = self.make_branch_and_tree('master')
+        tree = self.make_branch_and_tree('tree')
+        try:
+            tree.branch.bind(master_tree.branch)
+        except errors.UpgradeRequired:
+            # legacy branches cannot bind
+            return
+        master_tree.commit('foo', rev_id='foo', allow_pointless=True)
+        tree.update()
+        self.assertEqual('foo', tree.last_revision())
+        self.assertEqual('foo', tree.branch.last_revision())
+
+    def test_update_turns_local_commit_into_merge(self):
+        # doing an update with a few local commits and no master commits
+        # makes pending-merges. 
+        # this is done so that 'bzr update; bzr revert' will always produce
+        # an exact copy of the 'logical branch' - the referenced branch for
+        # a checkout, and the master for a bound branch.
+        # its possible that we should instead have 'bzr update' when there
+        # is nothing new on the master leave the current commits intact and
+        # alter 'revert' to revert to the master always. But for now, its
+        # good.
+        master_tree = self.make_branch_and_tree('master')
+        tree = self.make_branch_and_tree('tree')
+        try:
+            tree.branch.bind(master_tree.branch)
+        except errors.UpgradeRequired:
+            # legacy branches cannot bind
+            return
+        tree.commit('foo', rev_id='foo', allow_pointless=True, local=True)
+        tree.commit('bar', rev_id='bar', allow_pointless=True, local=True)
+        tree.update()
+        self.assertEqual(None, tree.last_revision())
+        self.assertEqual([], tree.branch.revision_history())
+        self.assertEqual(['bar'], tree.pending_merges())
+

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-02-22 05:45:47 +0000
+++ bzrlib/workingtree.py	2006-03-05 11:48:54 +0000
@@ -64,7 +64,9 @@
 from bzrlib.inventory import InventoryEntry, Inventory
 from bzrlib.lockable_files import LockableFiles
 from bzrlib.merge import merge_inner, transform_tree
-from bzrlib.osutils import (appendpath,
+from bzrlib.osutils import (
+                            abspath,
+                            appendpath,
                             compact_date,
                             file_kind,
                             isdir,
@@ -74,7 +76,6 @@
                             safe_unicode,
                             splitpath,
                             rand_bytes,
-                            abspath,
                             normpath,
                             realpath,
                             relpath,
@@ -1161,17 +1162,57 @@
 
     @needs_write_lock
     def update(self):
+        """Update a working tree along its branch.
+
+        This will update the branch if its bound too, which means we have multiple trees involved:
+        The new basis tree of the master.
+        The old basis tree of the branch.
+        The old basis tree of the working tree.
+        The current working tree state.
+        pathologically all three may be different, and non ancestors of each other.
+        Conceptually we want to:
+        Preserve the wt.basis->wt.state changes
+        Transform the wt.basis to the new master basis.
+        Apply a merge of the old branch basis to get any 'local' changes from it into the tree.
+        Restore the wt.basis->wt.state changes.
+
+        There isn't a single operation at the moment to do that, so we:
+        Merge current state -> basis tree of the master w.r.t. the old tree basis.
+        Do a 'normal' merge of the old branch basis if it is relevant.
+        """
+        old_tip = self.branch.update()
+        if old_tip is not None:
+            self.add_pending_merge(old_tip)
         self.branch.lock_read()
         try:
-            if self.last_revision() == self.branch.last_revision():
-                return
-            basis = self.basis_tree()
-            to_tree = self.branch.basis_tree()
-            result = merge_inner(self.branch,
-                                 to_tree,
-                                 basis,
-                                 this_tree=self)
-            self.set_last_revision(self.branch.last_revision())
+            result = 0
+            if self.last_revision() != self.branch.last_revision():
+                # merge tree state up to new branch tip.
+                basis = self.basis_tree()
+                to_tree = self.branch.basis_tree()
+                result += merge_inner(self.branch,
+                                      to_tree,
+                                      basis,
+                                      this_tree=self)
+                self.set_last_revision(self.branch.last_revision())
+            if old_tip and old_tip != self.last_revision():
+                # our last revision was not the prior branch last reivison
+                # and we have converted that last revision to a pending merge.
+                # base is somewhere between the branch tip now
+                # and the now pending merge
+                from bzrlib.revision import common_ancestor
+                try:
+                    base_rev_id = common_ancestor(self.branch.last_revision(),
+                                                  old_tip,
+                                                  self.branch.repository)
+                except errors.NoCommonAncestor:
+                    base_rev_id = None
+                base_tree = self.branch.repository.revision_tree(base_rev_id)
+                other_tree = self.branch.repository.revision_tree(old_tip)
+                result += merge_inner(self.branch,
+                                      other_tree,
+                                      base_tree,
+                                      this_tree=self)
             return result
         finally:
             self.branch.unlock()

